@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/* vim: set ts=8 sts=4 et sw=4 tw=99: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,6 +11,7 @@
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/FileUtils.h"
+#include "mozilla/IOBuffers.h"
 #include "mozilla/Logging.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/Services.h"
@@ -22,7 +23,6 @@
 #include "nsDebug.h"
 #include "nsIFile.h"
 #include "nsIFileURL.h"
-#include "nsIObserverService.h"
 #include "nsNetUtil.h"
 #include "nsPromiseFlatString.h"
 #include "nsProxyRelease.h"
@@ -31,11 +31,6 @@
 #include "nsZipArchive.h"
 #include "xpcpublic.h"
 
-#include "mozilla/dom/ContentChild.h"
-
-#undef DELAYED_STARTUP_TOPIC
-#define DELAYED_STARTUP_TOPIC "sessionstore-windows-restored"
-
 namespace mozilla {
 namespace {
 static LazyLogModule gURLLog("URLPreloader");
@@ -56,7 +51,7 @@ nsresult URLPreloader::CollectReports(nsIHandleReportCallback* aHandleReport,
                      ShallowSizeOfIncludingThis(MallocSizeOf),
                      "Memory used by the URL preloader service itself.");
 
-  for (const auto& elem : IterHash(mCachedURLs)) {
+  for (const auto& elem : mCachedURLs.Values()) {
     nsAutoCString pathName;
     pathName.Append(elem->mPath);
     // The backslashes will automatically be replaced with slashes in
@@ -68,19 +63,37 @@ nsresult URLPreloader::CollectReports(nsIHandleReportCallback* aHandleReport,
                          elem->TypeString(), pathName.get());
 
     aHandleReport->Callback(
-        EmptyCString(), path, KIND_HEAP, UNITS_BYTES,
+        ""_ns, path, KIND_HEAP, UNITS_BYTES,
         elem->SizeOfIncludingThis(MallocSizeOf),
-        NS_LITERAL_CSTRING("Memory used to hold cache data for files which "
-                           "have been read or pre-loaded during this session."),
+        nsLiteralCString("Memory used to hold cache data for files which "
+                         "have been read or pre-loaded during this session."),
         aData);
   }
 
   return NS_OK;
 }
 
+// static
+already_AddRefed<URLPreloader> URLPreloader::Create(bool* aInitialized) {
+  // The static APIs like URLPreloader::Read work in the child process because
+  // they fall back to a synchronous read. The actual preloader must be
+  // explicitly initialized, and this should only be done in the parent.
+  MOZ_RELEASE_ASSERT(XRE_IsParentProcess());
+
+  RefPtr<URLPreloader> preloader = new URLPreloader();
+  if (preloader->InitInternal().isOk()) {
+    *aInitialized = true;
+    RegisterWeakMemoryReporter(preloader);
+  } else {
+    *aInitialized = false;
+  }
+
+  return preloader.forget();
+}
+
 URLPreloader& URLPreloader::GetSingleton() {
   if (!sSingleton) {
-    sSingleton = new URLPreloader();
+    sSingleton = Create(&sInitialized);
     ClearOnShutdown(&sSingleton);
   }
 
@@ -91,16 +104,10 @@ bool URLPreloader::sInitialized = false;
 
 StaticRefPtr<URLPreloader> URLPreloader::sSingleton;
 
-URLPreloader::URLPreloader() {
-  if (InitInternal().isOk()) {
-    sInitialized = true;
-    RegisterWeakMemoryReporter(this);
-  }
-}
-
 URLPreloader::~URLPreloader() {
   if (sInitialized) {
     UnregisterWeakMemoryReporter(this);
+    sInitialized = false;
   }
 }
 
@@ -124,28 +131,20 @@ Result<Ok, nsresult> URLPreloader::InitInternal() {
   mResProto = do_QueryInterface(ph, &rv);
   MOZ_TRY(rv);
 
-  mChromeReg = services::GetChromeRegistryService();
+  mChromeReg = services::GetChromeRegistry();
   if (!mChromeReg) {
     return Err(NS_ERROR_UNEXPECTED);
   }
 
-  if (XRE_IsParentProcess()) {
-    nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
-
-    obs->AddObserver(this, DELAYED_STARTUP_TOPIC, false);
-
-    MOZ_TRY(NS_GetSpecialDirectory("ProfLDS", getter_AddRefs(mProfD)));
-  } else {
-    mStartupFinished = true;
-    mReaderInitialized = true;
-  }
+  MOZ_TRY(NS_GetSpecialDirectory("ProfLDS", getter_AddRefs(mProfD)));
 
   return Ok();
 }
 
 URLPreloader& URLPreloader::ReInitialize() {
-  sSingleton = new URLPreloader();
-
+  MOZ_ASSERT(sSingleton);
+  sSingleton = nullptr;
+  sSingleton = Create(&sInitialized);
   return *sSingleton;
 }
 
@@ -158,38 +157,38 @@ Result<nsCOMPtr<nsIFile>, nsresult> URLPreloader::GetCacheFile(
   nsCOMPtr<nsIFile> cacheFile;
   MOZ_TRY(mProfD->Clone(getter_AddRefs(cacheFile)));
 
-  MOZ_TRY(cacheFile->AppendNative(NS_LITERAL_CSTRING("startupCache")));
+  MOZ_TRY(cacheFile->AppendNative("startupCache"_ns));
   Unused << cacheFile->Create(nsIFile::DIRECTORY_TYPE, 0777);
 
-  MOZ_TRY(cacheFile->Append(NS_LITERAL_STRING("urlCache") + suffix));
+  MOZ_TRY(cacheFile->Append(u"urlCache"_ns + suffix));
 
-  return Move(cacheFile);
+  return std::move(cacheFile);
 }
 
 static const uint8_t URL_MAGIC[] = "mozURLcachev002";
 
 Result<nsCOMPtr<nsIFile>, nsresult> URLPreloader::FindCacheFile() {
   nsCOMPtr<nsIFile> cacheFile;
-  MOZ_TRY_VAR(cacheFile, GetCacheFile(NS_LITERAL_STRING(".bin")));
+  MOZ_TRY_VAR(cacheFile, GetCacheFile(u".bin"_ns));
 
   bool exists;
   MOZ_TRY(cacheFile->Exists(&exists));
   if (exists) {
-    MOZ_TRY(
-        cacheFile->MoveTo(nullptr, NS_LITERAL_STRING("urlCache-current.bin")));
+    MOZ_TRY(cacheFile->MoveTo(nullptr, u"urlCache-current.bin"_ns));
   } else {
-    MOZ_TRY(cacheFile->SetLeafName(NS_LITERAL_STRING("urlCache-current.bin")));
+    MOZ_TRY(cacheFile->SetLeafName(u"urlCache-current.bin"_ns));
     MOZ_TRY(cacheFile->Exists(&exists));
     if (!exists) {
       return Err(NS_ERROR_FILE_NOT_FOUND);
     }
   }
 
-  return Move(cacheFile);
+  return std::move(cacheFile);
 }
 
 Result<Ok, nsresult> URLPreloader::WriteCache() {
   MOZ_ASSERT(!NS_IsMainThread());
+  MOZ_DIAGNOSTIC_ASSERT(mStartupFinished);
 
   // The script preloader might call us a second time, if it has to re-write
   // its cache after a cache flush. We don't care about cache flushes, since
@@ -206,7 +205,7 @@ Result<Ok, nsresult> URLPreloader::WriteCache() {
   LOG(Debug, "Writing cache...");
 
   nsCOMPtr<nsIFile> cacheFile;
-  MOZ_TRY_VAR(cacheFile, GetCacheFile(NS_LITERAL_STRING("-new.bin")));
+  MOZ_TRY_VAR(cacheFile, GetCacheFile(u"-new.bin"_ns));
 
   bool exists;
   MOZ_TRY(cacheFile->Exists(&exists));
@@ -220,9 +219,9 @@ Result<Ok, nsresult> URLPreloader::WriteCache() {
                                         &fd.rwget()));
 
     nsTArray<URLEntry*> entries;
-    for (auto& entry : IterHash(mCachedURLs)) {
+    for (const auto& entry : mCachedURLs.Values()) {
       if (entry->mReadTime) {
-        entries.AppendElement(entry);
+        entries.AppendElement(entry.get());
       }
     }
 
@@ -241,7 +240,7 @@ Result<Ok, nsresult> URLPreloader::WriteCache() {
     MOZ_TRY(Write(fd, buf.Get(), buf.cursor()));
   }
 
-  MOZ_TRY(cacheFile->MoveTo(nullptr, NS_LITERAL_STRING("urlCache.bin")));
+  MOZ_TRY(cacheFile->MoveTo(nullptr, u"urlCache.bin"_ns));
 
   NS_DispatchToMainThread(
       NewRunnableMethod("URLPreloader::Cleanup", this, &URLPreloader::Cleanup));
@@ -302,15 +301,32 @@ Result<Ok, nsresult> URLPreloader::ReadCache(
 
       LOG(Debug, "Cached file: %s %s", key.TypeString(), key.mPath.get());
 
-      auto entry = mCachedURLs.LookupOrAdd(key, key);
+      // Don't bother doing anything else if the key didn't load correctly.
+      // We're going to throw it out right away, and it is possible that this
+      // leads to pendingURLs getting into a weird state.
+      if (buf.error()) {
+        return Err(NS_ERROR_UNEXPECTED);
+      }
+
+      auto entry = mCachedURLs.GetOrInsertNew(key, key);
       entry->mResultCode = NS_ERROR_NOT_INITIALIZED;
 
+      if (entry->isInList()) {
+#ifdef NIGHTLY_BUILD
+        MOZ_DIAGNOSTIC_ASSERT(pendingURLs.contains(entry),
+                              "Entry should be in pendingURLs");
+        MOZ_DIAGNOSTIC_ASSERT(key.mPath.Length() > 0,
+                              "Path should be non-empty");
+        MOZ_DIAGNOSTIC_ASSERT(false, "Entry should be new and not in any list");
+#endif
+        return Err(NS_ERROR_UNEXPECTED);
+      }
+
       pendingURLs.insertBack(entry);
     }
 
-    if (buf.error()) {
-      return Err(NS_ERROR_UNEXPECTED);
-    }
+    MOZ_RELEASE_ASSERT(!buf.error(),
+                       "We should have already bailed on an error");
 
     cleanup.release();
   }
@@ -319,9 +335,17 @@ Result<Ok, nsresult> URLPreloader::ReadCache(
 }
 
 void URLPreloader::BackgroundReadFiles() {
+  auto cleanup = MakeScopeExit([&]() {
+    auto lock = mReaderThread.Lock();
+    auto& readerThread = lock.ref();
+    NS_DispatchToMainThread(NewRunnableMethod(
+        "nsIThread::AsyncShutdown", readerThread, &nsIThread::AsyncShutdown));
+
+    readerThread = nullptr;
+  });
+
   Vector<nsZipCursor> cursors;
   LinkedList<URLEntry> pendingURLs;
-
   {
     MonitorAutoLock mal(mMonitor);
 
@@ -414,48 +438,55 @@ void URLPreloader::BackgroundReadFiles() {
 
   // We're done reading pending entries, so clear the list.
   pendingURLs.clear();
-
-  NS_DispatchToMainThread(NewRunnableMethod(
-      "nsIThread::Shutdown", mReaderThread, &nsIThread::Shutdown));
-  mReaderThread = nullptr;
 }
 
 void URLPreloader::BeginBackgroundRead() {
-  if (!mReaderThread && !mReaderInitialized && sInitialized) {
+  auto lock = mReaderThread.Lock();
+  auto& readerThread = lock.ref();
+  if (!readerThread && !mReaderInitialized && sInitialized) {
+    nsresult rv;
+    rv = NS_NewNamedThread("BGReadURLs", getter_AddRefs(readerThread));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return;
+    }
+
     nsCOMPtr<nsIRunnable> runnable =
         NewRunnableMethod("URLPreloader::BackgroundReadFiles", this,
                           &URLPreloader::BackgroundReadFiles);
-
-    Unused << NS_NewNamedThread("BGReadURLs", getter_AddRefs(mReaderThread),
-                                runnable);
+    rv = readerThread->Dispatch(runnable.forget(), NS_DISPATCH_NORMAL);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      // If we can't launch the task, just destroy the thread
+      readerThread = nullptr;
+      return;
+    }
   }
 }
 
-Result<const nsCString, nsresult> URLPreloader::ReadInternal(
-    const CacheKey& key, ReadType readType) {
-  if (mStartupFinished) {
+Result<nsCString, nsresult> URLPreloader::ReadInternal(const CacheKey& key,
+                                                       ReadType readType) {
+  if (mStartupFinished || !mReaderInitialized) {
     URLEntry entry(key);
 
     return entry.Read();
   }
 
-  auto entry = mCachedURLs.LookupOrAdd(key, key);
+  auto entry = mCachedURLs.GetOrInsertNew(key, key);
 
   entry->UpdateUsedTime();
 
   return entry->ReadOrWait(readType);
 }
 
-Result<const nsCString, nsresult> URLPreloader::ReadURIInternal(
-    nsIURI* uri, ReadType readType) {
+Result<nsCString, nsresult> URLPreloader::ReadURIInternal(nsIURI* uri,
+                                                          ReadType readType) {
   CacheKey key;
   MOZ_TRY_VAR(key, ResolveURI(uri));
 
   return ReadInternal(key, readType);
 }
 
-/* static */ Result<const nsCString, nsresult> URLPreloader::Read(
-    const CacheKey& key, ReadType readType) {
+/* static */ Result<nsCString, nsresult> URLPreloader::Read(const CacheKey& key,
+                                                            ReadType readType) {
   // If we're being called before the preloader has been initialized (i.e.,
   // before the profile has been initialized), just fall back to a synchronous
   // read. This happens when we're reading .ini and preference files that are
@@ -467,7 +498,7 @@ Result<const nsCString, nsresult> URLPreloader::ReadURIInternal(
   return GetSingleton().ReadInternal(key, readType);
 }
 
-/* static */ Result<const nsCString, nsresult> URLPreloader::ReadURI(
+/* static */ Result<nsCString, nsresult> URLPreloader::ReadURI(
     nsIURI* uri, ReadType readType) {
   if (!sInitialized) {
     return Err(NS_ERROR_NOT_INITIALIZED);
@@ -476,12 +507,12 @@ Result<const nsCString, nsresult> URLPreloader::ReadURIInternal(
   return GetSingleton().ReadURIInternal(uri, readType);
 }
 
-/* static */ Result<const nsCString, nsresult> URLPreloader::ReadFile(
+/* static */ Result<nsCString, nsresult> URLPreloader::ReadFile(
     nsIFile* file, ReadType readType) {
   return Read(CacheKey(file), readType);
 }
 
-/* static */ Result<const nsCString, nsresult> URLPreloader::Read(
+/* static */ Result<nsCString, nsresult> URLPreloader::Read(
     FileLocation& location, ReadType readType) {
   if (location.IsZip()) {
     if (location.GetBaseZip()) {
@@ -496,7 +527,7 @@ Result<const nsCString, nsresult> URLPreloader::ReadURIInternal(
   return ReadFile(file, readType);
 }
 
-/* static */ Result<const nsCString, nsresult> URLPreloader::ReadZip(
+/* static */ Result<nsCString, nsresult> URLPreloader::ReadZip(
     nsZipArchive* zip, const nsACString& path, ReadType readType) {
   // If the zip archive belongs to an Omnijar location, map it to a cache
   // entry, and cache it as normal. Otherwise, simply read the entry
@@ -580,14 +611,14 @@ Result<FileLocation, nsresult> URLPreloader::CacheKey::ToFileLocation() {
     nsCOMPtr<nsIFile> file;
     MOZ_TRY(NS_NewLocalFile(NS_ConvertUTF8toUTF16(mPath), false,
                             getter_AddRefs(file)));
-    return Move(FileLocation(file));
+    return FileLocation(file);
   }
 
   RefPtr<nsZipArchive> zip = Archive();
-  return Move(FileLocation(zip, mPath.get()));
+  return FileLocation(zip, mPath.get());
 }
 
-Result<const nsCString, nsresult> URLPreloader::URLEntry::Read() {
+Result<nsCString, nsresult> URLPreloader::URLEntry::Read() {
   FileLocation location;
   MOZ_TRY_VAR(location, ToFileLocation());
 
@@ -595,8 +626,8 @@ Result<const nsCString, nsresult> URLPreloader::URLEntry::Read() {
   return mData;
 }
 
-/* static */ Result<const nsCString, nsresult>
-URLPreloader::URLEntry::ReadLocation(FileLocation& location) {
+/* static */ Result<nsCString, nsresult> URLPreloader::URLEntry::ReadLocation(
+    FileLocation& location) {
   FileLocation::Data data;
   MOZ_TRY(location.GetData(data));
 
@@ -607,10 +638,10 @@ URLPreloader::URLEntry::ReadLocation(FileLocation& location) {
   result.SetLength(size);
   MOZ_TRY(data.Copy(result.BeginWriting(), size));
 
-  return Move(result);
+  return std::move(result);
 }
 
-Result<const nsCString, nsresult> URLPreloader::URLEntry::ReadOrWait(
+Result<nsCString, nsresult> URLPreloader::URLEntry::ReadOrWait(
     ReadType readType) {
   auto now = TimeStamp::Now();
   LOG(Info, "Reading %s\n", mPath.get());
@@ -643,20 +674,14 @@ Result<const nsCString, nsresult> URLPreloader::URLEntry::ReadOrWait(
   return res;
 }
 
-inline URLPreloader::CacheKey::CacheKey(InputBuffer& buffer) { Code(buffer); }
-
-nsresult URLPreloader::Observe(nsISupports* subject, const char* topic,
-                               const char16_t* data) {
-  nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
-  if (!strcmp(topic, DELAYED_STARTUP_TOPIC)) {
-    obs->RemoveObserver(this, DELAYED_STARTUP_TOPIC);
-    mStartupFinished = true;
-  }
-
-  return NS_OK;
+inline URLPreloader::CacheKey::CacheKey(InputBuffer& buffer) {
+  Code(buffer);
+  MOZ_DIAGNOSTIC_ASSERT(
+      mType == TypeAppJar || mType == TypeGREJar || mType == TypeFile,
+      "mType should be valid");
 }
 
-NS_IMPL_ISUPPORTS(URLPreloader, nsIObserver, nsIMemoryReporter)
+NS_IMPL_ISUPPORTS(URLPreloader, nsIMemoryReporter)
 
 #undef LOG
 
