@@ -14,31 +14,10 @@
 
 using namespace mozilla;
 
-/***************************************************************************/
-// static shared...
-
-// Note this is returning the hash of the bit pattern of the first part of the
-// nsID, not the hash of the pointer to the nsID.
-
-static PLDHashNumber HashIIDPtrKey(const void* key) {
-  uintptr_t v;
-  memcpy(&v, key, sizeof(v));
-  return HashGeneric(v);
-}
-
-static bool MatchIIDPtrKey(const PLDHashEntryHdr* entry, const void* key) {
-  return ((const nsID*)key)
-      ->Equals(*((const nsID*)((PLDHashEntryStub*)entry)->key));
-}
-
-static PLDHashNumber HashNativeKey(const void* data) {
-  return static_cast<const XPCNativeSetKey*>(data)->Hash();
-}
-
 /***************************************************************************/
 // implement JSObject2WrappedJSMap...
 
-void JSObject2WrappedJSMap::UpdateWeakPointersAfterGC() {
+void JSObject2WrappedJSMap::UpdateWeakPointersAfterGC(JSTracer* trc) {
   // Check all wrappers and update their JSObject pointer if it has been
   // moved. Release any wrappers whose weakly held JSObject has died.
 
@@ -50,7 +29,7 @@ void JSObject2WrappedJSMap::UpdateWeakPointersAfterGC() {
     // Walk the wrapper chain and update all JSObjects.
     while (wrapper) {
       if (wrapper->IsSubjectToFinalization()) {
-        wrapper->UpdateObjectPointerAfterGC();
+        wrapper->UpdateObjectPointerAfterGC(trc);
         if (!wrapper->GetJSObjectPreserveColor()) {
           dying.AppendElement(dont_AddRef(wrapper));
         }
@@ -59,12 +38,8 @@ void JSObject2WrappedJSMap::UpdateWeakPointersAfterGC() {
     }
 
     // Remove or update the JSObject key in the table if necessary.
-    JSObject* obj = iter.get().key().unbarrieredGet();
-    JS_UpdateWeakPointerAfterGCUnbarriered(&obj);
-    if (!obj) {
+    if (!JS_UpdateWeakPointerAfterGC(trc, &iter.get().mutableKey())) {
       iter.remove();
-    } else {
-      iter.get().mutableKey() = obj;
     }
   }
 }
@@ -97,21 +72,15 @@ size_t JSObject2WrappedJSMap::SizeOfWrappedJS(
 /***************************************************************************/
 // implement Native2WrappedNativeMap...
 
-// static
-Native2WrappedNativeMap* Native2WrappedNativeMap::newMap(int length) {
-  return new Native2WrappedNativeMap(length);
-}
-
-Native2WrappedNativeMap::Native2WrappedNativeMap(int length)
-    : mTable(PLDHashTable::StubOps(), sizeof(Entry), length) {}
+Native2WrappedNativeMap::Native2WrappedNativeMap()
+    : mMap(XPC_NATIVE_MAP_LENGTH) {}
 
 size_t Native2WrappedNativeMap::SizeOfIncludingThis(
     mozilla::MallocSizeOf mallocSizeOf) const {
   size_t n = mallocSizeOf(this);
-  n += mTable.ShallowSizeOfExcludingThis(mallocSizeOf);
-  for (auto iter = mTable.ConstIter(); !iter.Done(); iter.Next()) {
-    auto entry = static_cast<Native2WrappedNativeMap::Entry*>(iter.Get());
-    n += mallocSizeOf(entry->value);
+  n += mMap.shallowSizeOfExcludingThis(mallocSizeOf);
+  for (auto iter = mMap.iter(); !iter.done(); iter.next()) {
+    n += mallocSizeOf(iter.get().value());
   }
   return n;
 }
@@ -119,25 +88,15 @@ size_t Native2WrappedNativeMap::SizeOfIncludingThis(
 /***************************************************************************/
 // implement IID2NativeInterfaceMap...
 
-const struct PLDHashTableOps IID2NativeInterfaceMap::Entry::sOps = {
-    HashIIDPtrKey, MatchIIDPtrKey, PLDHashTable::MoveEntryStub,
-    PLDHashTable::ClearEntryStub};
-
-// static
-IID2NativeInterfaceMap* IID2NativeInterfaceMap::newMap(int length) {
-  return new IID2NativeInterfaceMap(length);
-}
-
-IID2NativeInterfaceMap::IID2NativeInterfaceMap(int length)
-    : mTable(&Entry::sOps, sizeof(Entry), length) {}
+IID2NativeInterfaceMap::IID2NativeInterfaceMap()
+    : mMap(XPC_NATIVE_INTERFACE_MAP_LENGTH) {}
 
 size_t IID2NativeInterfaceMap::SizeOfIncludingThis(
     mozilla::MallocSizeOf mallocSizeOf) const {
   size_t n = mallocSizeOf(this);
-  n += mTable.ShallowSizeOfExcludingThis(mallocSizeOf);
-  for (auto iter = mTable.ConstIter(); !iter.Done(); iter.Next()) {
-    auto entry = static_cast<IID2NativeInterfaceMap::Entry*>(iter.Get());
-    n += entry->value->SizeOfIncludingThis(mallocSizeOf);
+  n += mMap.shallowSizeOfExcludingThis(mallocSizeOf);
+  for (auto iter = mMap.iter(); !iter.done(); iter.next()) {
+    n += iter.get().value()->SizeOfIncludingThis(mallocSizeOf);
   }
   return n;
 }
@@ -145,61 +104,28 @@ size_t IID2NativeInterfaceMap::SizeOfIncludingThis(
 /***************************************************************************/
 // implement ClassInfo2NativeSetMap...
 
-// static
-bool ClassInfo2NativeSetMap::Entry::Match(const PLDHashEntryHdr* aEntry,
-                                          const void* aKey) {
-  return static_cast<const Entry*>(aEntry)->key == aKey;
-}
-
-// static
-void ClassInfo2NativeSetMap::Entry::Clear(PLDHashTable* aTable,
-                                          PLDHashEntryHdr* aEntry) {
-  auto entry = static_cast<Entry*>(aEntry);
-  NS_RELEASE(entry->value);
-
-  entry->key = nullptr;
-  entry->value = nullptr;
-}
-
-const PLDHashTableOps ClassInfo2NativeSetMap::Entry::sOps = {
-    PLDHashTable::HashVoidPtrKeyStub, Match, PLDHashTable::MoveEntryStub, Clear,
-    nullptr};
-
-// static
-ClassInfo2NativeSetMap* ClassInfo2NativeSetMap::newMap(int length) {
-  return new ClassInfo2NativeSetMap(length);
-}
-
-ClassInfo2NativeSetMap::ClassInfo2NativeSetMap(int length)
-    : mTable(&ClassInfo2NativeSetMap::Entry::sOps, sizeof(Entry), length) {}
+ClassInfo2NativeSetMap::ClassInfo2NativeSetMap()
+    : mMap(XPC_NATIVE_SET_MAP_LENGTH) {}
 
 size_t ClassInfo2NativeSetMap::ShallowSizeOfIncludingThis(
     mozilla::MallocSizeOf mallocSizeOf) {
   size_t n = mallocSizeOf(this);
-  n += mTable.ShallowSizeOfExcludingThis(mallocSizeOf);
+  n += mMap.shallowSizeOfExcludingThis(mallocSizeOf);
   return n;
 }
 
 /***************************************************************************/
 // implement ClassInfo2WrappedNativeProtoMap...
 
-// static
-ClassInfo2WrappedNativeProtoMap* ClassInfo2WrappedNativeProtoMap::newMap(
-    int length) {
-  return new ClassInfo2WrappedNativeProtoMap(length);
-}
-
-ClassInfo2WrappedNativeProtoMap::ClassInfo2WrappedNativeProtoMap(int length)
-    : mTable(PLDHashTable::StubOps(), sizeof(Entry), length) {}
+ClassInfo2WrappedNativeProtoMap::ClassInfo2WrappedNativeProtoMap()
+    : mMap(XPC_NATIVE_PROTO_MAP_LENGTH) {}
 
 size_t ClassInfo2WrappedNativeProtoMap::SizeOfIncludingThis(
     mozilla::MallocSizeOf mallocSizeOf) const {
   size_t n = mallocSizeOf(this);
-  n += mTable.ShallowSizeOfExcludingThis(mallocSizeOf);
-  for (auto iter = mTable.ConstIter(); !iter.Done(); iter.Next()) {
-    auto entry =
-        static_cast<ClassInfo2WrappedNativeProtoMap::Entry*>(iter.Get());
-    n += mallocSizeOf(entry->value);
+  n += mMap.shallowSizeOfExcludingThis(mallocSizeOf);
+  for (auto iter = mMap.iter(); !iter.done(); iter.next()) {
+    n += mallocSizeOf(iter.get().value());
   }
   return n;
 }
@@ -207,11 +133,12 @@ size_t ClassInfo2WrappedNativeProtoMap::SizeOfIncludingThis(
 /***************************************************************************/
 // implement NativeSetMap...
 
-bool NativeSetMap::Entry::Match(const PLDHashEntryHdr* entry, const void* key) {
-  auto Key = static_cast<const XPCNativeSetKey*>(key);
-  XPCNativeSet* SetInTable = ((Entry*)entry)->key_value;
-  XPCNativeSet* Set = Key->GetBaseSet();
-  XPCNativeInterface* Addition = Key->GetAddition();
+bool NativeSetHasher::match(Key key, Lookup lookup) {
+  // The |key| argument is for the existing table entry and |lookup| is the
+  // value passed by the caller that is being compared with it.
+  XPCNativeSet* SetInTable = key;
+  XPCNativeSet* Set = lookup->GetBaseSet();
+  XPCNativeInterface* Addition = lookup->GetAddition();
 
   if (!Set) {
     // This is a special case to deal with the invariant that says:
@@ -248,38 +175,16 @@ bool NativeSetMap::Entry::Match(const PLDHashEntryHdr* entry, const void* key) {
   return !Addition || Addition == *(CurrentInTable++);
 }
 
-const struct PLDHashTableOps NativeSetMap::Entry::sOps = {
-    HashNativeKey, Match, PLDHashTable::MoveEntryStub,
-    PLDHashTable::ClearEntryStub};
-
-// static
-NativeSetMap* NativeSetMap::newMap(int length) {
-  return new NativeSetMap(length);
-}
-
-NativeSetMap::NativeSetMap(int length)
-    : mTable(&Entry::sOps, sizeof(Entry), length) {}
+NativeSetMap::NativeSetMap() : mSet(XPC_NATIVE_SET_MAP_LENGTH) {}
 
 size_t NativeSetMap::SizeOfIncludingThis(
     mozilla::MallocSizeOf mallocSizeOf) const {
   size_t n = mallocSizeOf(this);
-  n += mTable.ShallowSizeOfExcludingThis(mallocSizeOf);
-  for (auto iter = mTable.ConstIter(); !iter.Done(); iter.Next()) {
-    auto entry = static_cast<NativeSetMap::Entry*>(iter.Get());
-    n += entry->key_value->SizeOfIncludingThis(mallocSizeOf);
+  n += mSet.shallowSizeOfExcludingThis(mallocSizeOf);
+  for (auto iter = mSet.iter(); !iter.done(); iter.next()) {
+    n += iter.get()->SizeOfIncludingThis(mallocSizeOf);
   }
   return n;
 }
 
 /***************************************************************************/
-// implement XPCWrappedNativeProtoMap...
-
-// static
-XPCWrappedNativeProtoMap* XPCWrappedNativeProtoMap::newMap(int length) {
-  return new XPCWrappedNativeProtoMap(length);
-}
-
-XPCWrappedNativeProtoMap::XPCWrappedNativeProtoMap(int length)
-    : mTable(PLDHashTable::StubOps(), sizeof(PLDHashEntryStub), length) {}
-
-/***************************************************************************/
