@@ -6,6 +6,7 @@
 
 /* Class used to manage the wrapped native objects within a JS scope. */
 
+#include "AccessCheck.h"
 #include "xpcprivate.h"
 #include "XPCWrapper.h"
 #include "nsContentUtils.h"
@@ -15,8 +16,10 @@
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Unused.h"
-#include "nsIXULRuntime.h"
 #include "mozJSComponentLoader.h"
+#include "js/Object.h"              // JS::GetCompartment
+#include "js/PropertyAndElement.h"  // JS_DefineProperty, JS_DefinePropertyById
+#include "js/RealmIterators.h"
 
 #include "mozilla/dom/BindingUtils.h"
 
@@ -64,9 +67,9 @@ static bool RemoteXULForbidsXBLScope(HandleObject aFirstGlobal) {
 
 XPCWrappedNativeScope::XPCWrappedNativeScope(JS::Compartment* aCompartment,
                                              JS::HandleObject aFirstGlobal)
-    : mWrappedNativeMap(Native2WrappedNativeMap::newMap(XPC_NATIVE_MAP_LENGTH)),
+    : mWrappedNativeMap(mozilla::MakeUnique<Native2WrappedNativeMap>()),
       mWrappedNativeProtoMap(
-          ClassInfo2WrappedNativeProtoMap::newMap(XPC_NATIVE_PROTO_MAP_LENGTH)),
+          mozilla::MakeUnique<ClassInfo2WrappedNativeProtoMap>()),
       mComponents(nullptr),
       mCompartment(aCompartment) {
 #ifdef DEBUG
@@ -84,8 +87,11 @@ XPCWrappedNativeScope::XPCWrappedNativeScope(JS::Compartment* aCompartment,
   // remote XUL domains, _except_ if we have an additional pref override set.
   //
   // Note that we can't quite remove this yet, even though we never actually
-  // use XBL scopes, because some code (including the security manager) uses
-  // this boolean to make decisions that we rely on in our test infrastructure.
+  // use XBL scopes, because the security manager uses this boolean to make
+  // decisions that we rely on in our test infrastructure.
+  //
+  // FIXME(emilio): Now that the security manager is the only caller probably
+  // should be renamed, but what's a good name for this?
   mAllowContentXBLScope = !RemoteXULForbidsXBLScope(aFirstGlobal);
 }
 
@@ -93,8 +99,8 @@ bool XPCWrappedNativeScope::GetComponentsJSObject(JSContext* cx,
                                                   JS::MutableHandleObject obj) {
   if (!mComponents) {
     bool system = AccessCheck::isChrome(mCompartment);
-    mComponents =
-        system ? new nsXPCComponents(this) : new nsXPCComponentsBase(this);
+    MOZ_RELEASE_ASSERT(system, "How did we get a non-system Components?");
+    mComponents = new nsXPCComponents(this);
   }
 
   RootedValue val(cx);
@@ -109,22 +115,10 @@ bool XPCWrappedNativeScope::GetComponentsJSObject(JSContext* cx,
     return false;
   }
 
-  // The call to wrap() here is necessary even though the object is same-
-  // compartment, because it applies our security wrapper.
   obj.set(&val.toObject());
-  if (NS_WARN_IF(!JS_WrapObject(cx, obj))) {
-    return false;
-  }
   return true;
 }
 
-void XPCWrappedNativeScope::ForcePrivilegedComponents() {
-  nsCOMPtr<nsIXPCComponents> c = do_QueryInterface(mComponents);
-  if (!c) {
-    mComponents = new nsXPCComponents(this);
-  }
-}
-
 static bool DefineSubcomponentProperty(JSContext* aCx, HandleObject aGlobal,
                                        nsISupports* aSubcomponent,
                                        const nsID* aIID,
@@ -149,14 +143,7 @@ bool XPCWrappedNativeScope::AttachComponentsObject(JSContext* aCx) {
 
   RootedObject global(aCx, CurrentGlobalOrNull(aCx));
 
-  // The global Components property is non-configurable if it's a full
-  // nsXPCComponents object. That way, if it's an nsXPCComponentsBase,
-  // enableUniversalXPConnect can upgrade it later.
-  unsigned attrs = JSPROP_READONLY | JSPROP_RESOLVING;
-  nsCOMPtr<nsIXPCComponents> c = do_QueryInterface(mComponents);
-  if (c) {
-    attrs |= JSPROP_PERMANENT;
-  }
+  const unsigned attrs = JSPROP_READONLY | JSPROP_RESOLVING | JSPROP_PERMANENT;
 
   RootedId id(aCx,
               XPCJSContext::Get()->GetStringID(XPCJSContext::IDX_COMPONENTS));
@@ -175,32 +162,15 @@ bool XPCWrappedNativeScope::AttachComponentsObject(JSContext* aCx) {
   DEFINE_SUBCOMPONENT_PROPERTY(mComponents, Interfaces, nullptr, CI)
   DEFINE_SUBCOMPONENT_PROPERTY(mComponents, Results, nullptr, CR)
 
-  if (!c) {
-    return true;
-  }
-
-  DEFINE_SUBCOMPONENT_PROPERTY(c, Classes, nullptr, CC)
-  DEFINE_SUBCOMPONENT_PROPERTY(c, Utils, &NS_GET_IID(nsIXPCComponents_Utils),
-                               CU)
+  DEFINE_SUBCOMPONENT_PROPERTY(mComponents, Classes, nullptr, CC)
+  DEFINE_SUBCOMPONENT_PROPERTY(mComponents, Utils,
+                               &NS_GET_IID(nsIXPCComponents_Utils), CU)
 
 #undef DEFINE_SUBCOMPONENT_PROPERTY
 
   return true;
 }
 
-JSObject* XPCWrappedNativeScope::EnsureContentXBLScope(JSContext* cx) {
-  JS::RootedObject global(cx, CurrentGlobalOrNull(cx));
-  MOZ_ASSERT(js::IsObjectInContextCompartment(global, cx));
-  MOZ_ASSERT(!IsContentXBLScope());
-  MOZ_ASSERT(strcmp(js::GetObjectClass(global)->name,
-                    "nsXBLPrototypeScript compilation scope"));
-
-  // We can probably remove EnsureContentXBLScope and clean up all its callers,
-  // but a bunch (all?) of those callers will just go away when we remove XBL
-  // support, so it's simpler to just leave it here as a no-op.
-  return global;
-}
-
 bool XPCWrappedNativeScope::XBLScopeStateMatches(nsIPrincipal* aPrincipal) {
   return mAllowContentXBLScope ==
          !RemoteXULForbidsXBLScopeForPrincipal(aPrincipal);
@@ -214,25 +184,12 @@ bool XPCWrappedNativeScope::AllowContentXBLScope(Realm* aRealm) {
 }
 
 namespace xpc {
-JSObject* GetXBLScope(JSContext* cx, JSObject* contentScopeArg) {
-  JS::RootedObject contentScope(cx, contentScopeArg);
-  JSAutoRealm ar(cx, contentScope);
-  XPCWrappedNativeScope* nativeScope = ObjectScope(contentScope);
-
-  RootedObject scope(cx, nativeScope->EnsureContentXBLScope(cx));
-  NS_ENSURE_TRUE(scope, nullptr);  // See bug 858642.
-
-  scope = js::UncheckedUnwrap(scope);
-  JS::ExposeObjectToActiveJS(scope);
-  return scope;
-}
-
 JSObject* GetUAWidgetScope(JSContext* cx, JSObject* contentScopeArg) {
   JS::RootedObject contentScope(cx, contentScopeArg);
   JSAutoRealm ar(cx, contentScope);
   nsIPrincipal* principal = GetObjectPrincipal(contentScope);
 
-  if (nsContentUtils::IsSystemPrincipal(principal)) {
+  if (principal->IsSystemPrincipal()) {
     return JS::GetNonCCWObjectGlobal(contentScope);
   }
 
@@ -263,10 +220,8 @@ XPCWrappedNativeScope::~XPCWrappedNativeScope() {
   // We can do additional cleanup assertions here...
 
   MOZ_ASSERT(0 == mWrappedNativeMap->Count(), "scope has non-empty map");
-  delete mWrappedNativeMap;
 
   MOZ_ASSERT(0 == mWrappedNativeProtoMap->Count(), "scope has non-empty map");
-  delete mWrappedNativeProtoMap;
 
   // This should not be necessary, since the Components object should die
   // with the scope but just in case.
@@ -293,9 +248,8 @@ void XPCWrappedNativeScope::TraceWrappedNativesInAllScopes(XPCJSRuntime* xpcrt,
   // AllScopes() because that asserts we're on the main thread.
 
   for (XPCWrappedNativeScope* cur : xpcrt->GetWrappedNativeScopes()) {
-    for (auto i = cur->mWrappedNativeMap->Iter(); !i.Done(); i.Next()) {
-      auto entry = static_cast<Native2WrappedNativeMap::Entry*>(i.Get());
-      XPCWrappedNative* wrapper = entry->value;
+    for (auto i = cur->mWrappedNativeMap->Iter(); !i.done(); i.next()) {
+      XPCWrappedNative* wrapper = i.get().value();
       if (wrapper->HasExternalReference() && !wrapper->IsWrapperExpired()) {
         wrapper->TraceSelf(trc);
       }
@@ -307,16 +261,16 @@ void XPCWrappedNativeScope::TraceWrappedNativesInAllScopes(XPCJSRuntime* xpcrt,
 void XPCWrappedNativeScope::SuspectAllWrappers(
     nsCycleCollectionNoteRootCallback& cb) {
   for (XPCWrappedNativeScope* cur : AllScopes()) {
-    for (auto i = cur->mWrappedNativeMap->Iter(); !i.Done(); i.Next()) {
-      static_cast<Native2WrappedNativeMap::Entry*>(i.Get())->value->Suspect(cb);
+    for (auto i = cur->mWrappedNativeMap->Iter(); !i.done(); i.next()) {
+      i.get().value()->Suspect(cb);
     }
   }
 }
 
-void XPCWrappedNativeScope::UpdateWeakPointersAfterGC() {
+void XPCWrappedNativeScope::UpdateWeakPointersAfterGC(JSTracer* trc) {
   // Sweep waivers.
   if (mWaiverWrapperMap) {
-    mWaiverWrapperMap->Sweep();
+    mWaiverWrapperMap->UpdateWeakPointers(trc);
   }
 
   if (!js::IsCompartmentZoneSweepingOrCompacting(mCompartment)) {
@@ -341,28 +295,28 @@ void XPCWrappedNativeScope::UpdateWeakPointersAfterGC() {
 
   // Sweep mWrappedNativeMap for dying flat JS objects. Moving has already
   // been handled by XPCWrappedNative::FlatJSObjectMoved.
-  for (auto iter = GetWrappedNativeMap()->Iter(); !iter.Done(); iter.Next()) {
-    auto entry = static_cast<Native2WrappedNativeMap::Entry*>(iter.Get());
-    XPCWrappedNative* wrapper = entry->value;
+  for (auto iter = GetWrappedNativeMap()->ModIter(); !iter.done();
+       iter.next()) {
+    XPCWrappedNative* wrapper = iter.get().value();
     JSObject* obj = wrapper->GetFlatJSObjectPreserveColor();
-    JS_UpdateWeakPointerAfterGCUnbarriered(&obj);
-    MOZ_ASSERT(!obj || obj == wrapper->GetFlatJSObjectPreserveColor());
-    MOZ_ASSERT_IF(obj, js::GetObjectCompartment(obj) == mCompartment);
-    if (!obj) {
-      iter.Remove();
+    if (JS_UpdateWeakPointerAfterGCUnbarriered(trc, &obj)) {
+      MOZ_ASSERT(obj == wrapper->GetFlatJSObjectPreserveColor());
+      MOZ_ASSERT(JS::GetCompartment(obj) == mCompartment);
+    } else {
+      iter.remove();
     }
   }
 
   // Sweep mWrappedNativeProtoMap for dying prototype JSObjects. Moving has
   // already been handled by XPCWrappedNativeProto::JSProtoObjectMoved.
-  for (auto i = mWrappedNativeProtoMap->Iter(); !i.Done(); i.Next()) {
-    auto entry = static_cast<ClassInfo2WrappedNativeProtoMap::Entry*>(i.Get());
-    JSObject* obj = entry->value->GetJSProtoObjectPreserveColor();
-    JS_UpdateWeakPointerAfterGCUnbarriered(&obj);
-    MOZ_ASSERT_IF(obj, js::GetObjectCompartment(obj) == mCompartment);
-    MOZ_ASSERT(!obj || obj == entry->value->GetJSProtoObjectPreserveColor());
-    if (!obj) {
-      i.Remove();
+  for (auto i = mWrappedNativeProtoMap->ModIter(); !i.done(); i.next()) {
+    XPCWrappedNativeProto* proto = i.get().value();
+    JSObject* obj = proto->GetJSProtoObjectPreserveColor();
+    if (JS_UpdateWeakPointerAfterGCUnbarriered(trc, &obj)) {
+      MOZ_ASSERT(JS::GetCompartment(obj) == mCompartment);
+      MOZ_ASSERT(obj == proto->GetJSProtoObjectPreserveColor());
+    } else {
+      i.remove();
     }
   }
 }
@@ -370,9 +324,8 @@ void XPCWrappedNativeScope::UpdateWeakPointersAfterGC() {
 // static
 void XPCWrappedNativeScope::SweepAllWrappedNativeTearOffs() {
   for (XPCWrappedNativeScope* cur : AllScopes()) {
-    for (auto i = cur->mWrappedNativeMap->Iter(); !i.Done(); i.Next()) {
-      auto entry = static_cast<Native2WrappedNativeMap::Entry*>(i.Get());
-      entry->value->SweepTearOffs();
+    for (auto i = cur->mWrappedNativeMap->Iter(); !i.done(); i.next()) {
+      i.get().value()->SweepTearOffs();
     }
   }
 }
@@ -402,19 +355,13 @@ void XPCWrappedNativeScope::SystemIsBeingShutDown() {
 
     // Walk the protos first. Wrapper shutdown can leave dangling
     // proto pointers in the proto map.
-    for (auto i = cur->mWrappedNativeProtoMap->Iter(); !i.Done(); i.Next()) {
-      auto entry =
-          static_cast<ClassInfo2WrappedNativeProtoMap::Entry*>(i.Get());
-      entry->value->SystemIsBeingShutDown();
-      i.Remove();
+    for (auto i = cur->mWrappedNativeProtoMap->ModIter(); !i.done(); i.next()) {
+      i.get().value()->SystemIsBeingShutDown();
+      i.remove();
     }
-    for (auto i = cur->mWrappedNativeMap->Iter(); !i.Done(); i.Next()) {
-      auto entry = static_cast<Native2WrappedNativeMap::Entry*>(i.Get());
-      XPCWrappedNative* wrapper = entry->value;
-      if (wrapper->IsValid()) {
-        wrapper->SystemIsBeingShutDown();
-      }
-      i.Remove();
+    for (auto i = cur->mWrappedNativeMap->ModIter(); !i.done(); i.next()) {
+      i.get().value()->SystemIsBeingShutDown();
+      i.remove();
     }
 
     CompartmentPrivate* priv = CompartmentPrivate::Get(cur->Compartment());
@@ -486,26 +433,24 @@ void XPCWrappedNativeScope::DebugDump(int16_t depth) {
   XPC_LOG_ALWAYS(("mCompartment @ %p", mCompartment));
 
   XPC_LOG_ALWAYS(("mWrappedNativeMap @ %p with %d wrappers(s)",
-                  mWrappedNativeMap, mWrappedNativeMap->Count()));
+                  mWrappedNativeMap.get(), mWrappedNativeMap->Count()));
   // iterate contexts...
   if (depth && mWrappedNativeMap->Count()) {
     XPC_LOG_INDENT();
-    for (auto i = mWrappedNativeMap->Iter(); !i.Done(); i.Next()) {
-      auto entry = static_cast<Native2WrappedNativeMap::Entry*>(i.Get());
-      entry->value->DebugDump(depth);
+    for (auto i = mWrappedNativeMap->Iter(); !i.done(); i.next()) {
+      i.get().value()->DebugDump(depth);
     }
     XPC_LOG_OUTDENT();
   }
 
   XPC_LOG_ALWAYS(("mWrappedNativeProtoMap @ %p with %d protos(s)",
-                  mWrappedNativeProtoMap, mWrappedNativeProtoMap->Count()));
+                  mWrappedNativeProtoMap.get(),
+                  mWrappedNativeProtoMap->Count()));
   // iterate contexts...
   if (depth && mWrappedNativeProtoMap->Count()) {
     XPC_LOG_INDENT();
-    for (auto i = mWrappedNativeProtoMap->Iter(); !i.Done(); i.Next()) {
-      auto entry =
-          static_cast<ClassInfo2WrappedNativeProtoMap::Entry*>(i.Get());
-      entry->value->DebugDump(depth);
+    for (auto i = mWrappedNativeProtoMap->Iter(); !i.done(); i.next()) {
+      i.get().value()->DebugDump(depth);
     }
     XPC_LOG_OUTDENT();
   }
@@ -528,7 +473,8 @@ void XPCWrappedNativeScope::AddSizeOfIncludingThis(
   scopeSizeInfo->mScopeAndMapSize +=
       mWrappedNativeProtoMap->SizeOfIncludingThis(scopeSizeInfo->mMallocSizeOf);
 
-  auto realmCb = [](JSContext*, void* aData, JS::Handle<JS::Realm*> aRealm) {
+  auto realmCb = [](JSContext*, void* aData, JS::Realm* aRealm,
+                    const JS::AutoRequireNoGC& nogc) {
     auto* scopeSizeInfo = static_cast<ScopeSizeInfo*>(aData);
     JSObject* global = GetRealmGlobalOrNull(aRealm);
     if (global && dom::HasProtoAndIfaceCache(global)) {
